import { MIXINS } from './Constants';
class Mixin {
    static connect(component) {
        throw new Error('static connect(component: WebComponent), must be redefined');
    }
    static disconnect(component) {
        throw new Error('static disconnect(component: WebComponent), must be redefined');
    }
    // must re-define
    static html(html) {
        throw new Error('static html(html: HTMLTemplateElement): void, must be redefined');
    }
    ;
    static css(css) {
        throw new Error('static css(css: HTMLTemplateElement): void, must be redefined');
    }
    ;
    static properties(properties) {
        throw new Error('static properties(properties: Map<string, string>): void, must be redefined');
    }
    ;
    static observed(observed) {
        throw new Error('static observed(observed: string[]): void, must be redefined');
    }
    ;
    static converters(converters) {
        throw new Error('static converters(converters: Map<string, (value: string) => any>): void, must be redefined');
    }
    ;
    // special members
    static register() {
        const id = this.id;
        if (!id) {
            throw new Error('No id was defined for this mixin');
        }
        if (MIXINS.has(id)) {
            throw new Error(`The mixin id ${id} has already been registered`);
        }
        MIXINS.set(id, this);
    }
    static stateOf(component) {
        const key = `@wayang/mixin/state/${this.id}`;
        if (!component.hasOwnProperty(key)) {
            component[key] = new Map();
        }
        return component[key];
    }
    static attach(component) {
        const cache = component.cache;
        if (cache.mixins.has(this.id)) {
            return;
        }
        cache.mixins.set(this.id, this);
        if (!component.mixins.includes(this)) {
            component.mixins.push(this);
        }
        this.html(cache.template);
        this.css(cache.styles);
        this.properties(cache.properties);
        this.observed(cache.observed);
        this.converters(cache.converters);
    }
}
export default Mixin;
