import { PLUGINS } from './Constants';
class Plugin {
    static connect(element) {
        throw new Error('static connect(element: Wayang), must be redefined');
    }
    static disconnect(element) {
        throw new Error('static disconnect(element: Wayang), must be redefined');
    }
    // must re-define
    static html(html) {
        throw new Error('static html(html: HTMLTemplateElement): void, must be redefined');
    }
    ;
    static css(css) {
        throw new Error('static css(css: HTMLTemplateElement): void, must be redefined');
    }
    ;
    static properties(properties) {
        throw new Error('static properties(properties: Map<string, string>): void, must be redefined');
    }
    ;
    static observed(observed) {
        throw new Error('static observed(observed: string[]): void, must be redefined');
    }
    ;
    static converters(converters) {
        throw new Error('static converters(converters: Map<string, (string) => any>): void, must be redefined');
    }
    ;
    // special members
    static register() {
        const id = this.id;
        if (!id) {
            throw new Error('No id was defined for this plugin');
        }
        if (PLUGINS.has(id)) {
            throw new Error(`The plugin id ${id} has already been registered`);
        }
        PLUGINS.set(id, this);
    }
    static attach(element) {
        const cache = element.cache;
        if (cache.plugins.has(this.id)) {
            return;
        }
        cache.plugins.set(this.id, this);
        if (!element.plugins.includes(this)) {
            element.plugins.push(this);
        }
        this.html(cache.template);
        this.css(cache.styles);
        this.properties(cache.properties);
        this.observed(cache.observed);
        this.converters(new Map(element.converters));
    }
}
export default Plugin;
