import global from 'whenthough';
import { PRESENTERS } from './Constants';
const instanceKey = Symbol.for(`@wayang/presenter/instance`);
class Presenter {
    constructor() {
        this.global = global;
    }
    static register() {
        // basic validation
        if (!this.id) {
            throw new Error('No id was defined for this presenter');
        }
        if (PRESENTERS.has(this.id)) {
            throw new Error(`The presenter id ${this.id} has already been registered`);
        }
        // registration
        PRESENTERS.set(this.id, this);
        global.set(Symbol.for(`@wayang/presenters/${this.id}`), this);
    }
    static new() {
        return new this();
    }
    static addTo(component, id) {
        global
            .request(Symbol.for(`@wayang/presenters/${id}`))
            .then(presenter => {
            if (component[instanceKey]) {
                return;
            }
            component[instanceKey] = presenter.new();
            component[instanceKey].connect(component);
        });
    }
    static removeFrom(component) {
        if (!component[instanceKey]) {
            return;
        }
        component[instanceKey].disconnect(component);
        component[instanceKey] = null;
    }
}
export default Presenter;
